@@ -1,5 +1,6 @@
 """Forms for FIDO 2 registration and login."""
 import base64
+from typing import Optional
 
 from django import forms
 from django.contrib.auth import authenticate
@@ -11,6 +12,8 @@ from fido2.client import ClientData
 from fido2.ctap2 import AttestationObject, AuthenticatorData
 from fido2.server import Fido2Server
 
+from .settings import SETTINGS
+
 
 class Fido2RegistrationForm(forms.Form):
     """Form for FIDO 2 registration responses."""
@@ -19,6 +22,7 @@ class Fido2RegistrationForm(forms.Form):
                                   widget=forms.HiddenInput)
     attestation = forms.CharField(error_messages={'required': _("Operation wasn't completed.")},
                                   widget=forms.HiddenInput)
+    user_handle = forms.CharField(widget=forms.HiddenInput)
     label = forms.CharField(required=False, max_length=255, label=_("Label"))
 
     class Media:
@@ -42,6 +46,17 @@ class Fido2RegistrationForm(forms.Form):
         except ValueError:
             raise ValidationError(_('FIDO 2 response is malformed.'), code='invalid')
 
+    def clean_user_handle(self) -> Optional[str]:
+        """Return decoded attestation object."""
+        if not SETTINGS.resident_key:
+            return None
+
+        value = self.cleaned_data['user_handle']
+        try:
+            return base64.b64decode(value).decode('utf-8')
+        except ValueError:
+            raise ValidationError(_('FIDO 2 response is malformed.'), code='invalid')
+
 
 class Fido2AuthenticationForm(forms.Form):
     """Form for FIDO 2 authentication responses."""
