@@ -9,6 +9,8 @@ USER_FULL_NAME = USER_FIRST_NAME + ' ' + USER_LAST_NAME
 REGISTRATION_CHALLENGE = "Rimmerstasks"
 REGISTRATION_CLIENT_DATA = ('eyJ0eXBlIjogIndlYmF1dGhuLmNyZWF0ZSIsICJjbGllbnRFeHRlbnNpb25zIjoge30sICJjaGFsbGVuZ2UiOiAiUm'
                             'ltbWVyc3Rhc2tzIiwgIm9yaWdpbiI6ICJodHRwczovL3Rlc3RzZXJ2ZXIifQ==')
+USER_HANDLE = 'ecc1479fbb8540a6a22fa655ab4c4baf'
+USER_HANDLE_B64 = 'ZWNjMTQ3OWZiYjg1NDBhNmEyMmZhNjU1YWI0YzRiYWY='
 ATTESTATION_OBJECT = (
     'owFoZmlkby11MmYCWMTr2rckepjnyU3pMGbSTDiv2aDKKY42hLR6uxEx/MnyAEEAAAAAAAAAAAAAAAAAAAAAAAAAAABAVKZJP1j2MAl+NE9Ho0pa+T'
     'ybzzmoI4XbU/MvHCr7lQBJc3O5WvHF9B85RIk2ZEyRSwVHh8MW+te50I9juwr6gqUBAgMmIAEhWCDhTkbEO9WpfLwhmoLoPnka4fFANkMYCqhpxI6V'
