@@ -1,6 +1,7 @@
 """Views for FIDO 2 registration and login."""
 import base64
 import logging
+import uuid
 import warnings
 from abc import ABCMeta, abstractmethod
 from enum import Enum, unique
@@ -18,7 +19,7 @@ from django.forms import Form
 from django.http import HttpRequest, HttpResponse, JsonResponse
 from django.shortcuts import redirect
 from django.urls import reverse_lazy
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.utils.translation import gettext_lazy as _
 from django.views.generic import FormView, View
 from fido2.attestation import Attestation, UnsupportedType
@@ -157,8 +158,8 @@ class BaseFido2RequestView(Fido2ViewMixin, View, metaclass=ABCMeta):
         except ValueError as error:
             return JsonResponse({
                 'error_code': getattr(error, 'error_code', Fido2ServerError.DEFAULT),
-                'message': force_text(error),
-                'error': force_text(error),  # error key is deprecated and will be removed in the future
+                'message': force_str(error),
+                'error': force_str(error),  # error key is deprecated and will be removed in the future
             }, status=BAD_REQUEST)
 
         # Encode challenge into base64 encoding
@@ -204,7 +205,11 @@ class Fido2RegistrationRequestView(LoginRequiredMixin, BaseFido2RequestView):
         In such case, it is required to provide another identifier which would differentiate users.
         See https://www.w3.org/TR/webauthn/#dom-publickeycredentialuserentity-id and
         https://tools.ietf.org/html/rfc8266#section-6.1 for details.
+
+        If resident_key is True, we need to return an uuid string that does not disclose user identity
         """
+        if SETTINGS.resident_key:
+            return uuid.uuid4().hex
         return user.username
 
     def get_user_data(self, user: AbstractBaseUser) -> Dict[str, str]:
@@ -224,7 +229,7 @@ class Fido2RegistrationRequestView(LoginRequiredMixin, BaseFido2RequestView):
         assert user.is_authenticated, "User must not be anonymous for FIDO 2 requests."
         credentials = self.get_credentials(user)
         return self.server.register_begin(self.get_user_data(user), credentials,
-                                          user_verification=self.user_verification)
+                                          user_verification=self.user_verification, resident_key=SETTINGS.resident_key)
 
 
 class Fido2RegistrationView(LoginRequiredMixin, Fido2ViewMixin, FormView):
@@ -278,6 +283,7 @@ class Fido2RegistrationView(LoginRequiredMixin, Fido2ViewMixin, FormView):
             return self.form_invalid(form)
 
         Authenticator.objects.create(user=self.request.user, attestation=form.cleaned_data['attestation'],
+                                     user_handle=form.cleaned_data.get('user_handle'),
                                      label=form.cleaned_data.get('label'))
         return super().form_valid(form)
 
